------------------------------------------------------------------------------
-- SellableItemDrops - Logging your sellable item drops
------------------------------------------------------------------------------
-- Modules/GUI_session.lua - Interface/Session Tab
--
-- Author: Expelliarm5s / November 2020 / All Rights Reserved
--
-- Version 1.4.20
------------------------------------------------------------------------------
-- luacheck: ignore 212

local addonName, addon = ...
local GUI = addon:GetModule("GUI")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local AceGUI = LibStub("AceGUI-3.0")
local private = {}
------------------------------------------------------------------------------

GUI.guiTabs.texts["session"] = {
	title=L["Session"], desc=L["Current Session"]}

GUI.guiTabs.order[#GUI.guiTabs.order + 1] = "session"

------------------------------------------------------------------------------
-- Session Tab

-- create session tab
GUI.guiTabs.select["session"] = function(container, group)
	GUI:DebugPrintf("Select Tab %s: %s", group, tostring(container))
	if not container then
		GUI:DebugPrintf("Select Tab: container is nil")
	end

	-- min. Size
	GUI.display.frame:SetMinResize(350, 240)

	-- Filter: default filter is empty
	GUI.sesFilters = {}

	-- variants
	local flNotTheRealItemString = ""
	if (addon.db.global.useNotTheRealItemString) then
		flNotTheRealItemString = ITEM_QUALITY_COLORS[5].hex .. L[" (! = variant)"] .. FONT_COLOR_CODE_CLOSE
	end

	if (CanIMogIt) then
		flNotTheRealItemString = flNotTheRealItemString .. " " .. ITEM_QUALITY_COLORS[5].hex .. L[" (TM = New Transmog)"] .. FONT_COLOR_CODE_CLOSE
	end

	local nextLapPopupText = L["Click to start a new lap."]
	if (addon.db.global.lpInstanceReset) then
		nextLapPopupText = L["Click to start a new lap and reset all instances."]
	end

	-- session control group
	local grp1
	if not GUI.isShrinked then
		grp1 = AceGUI:Create("SIDInlineGroup")
		grp1:SetLayout("Flow")
		if grp1.SetTitle then
			grp1:SetTitle(L["Session Control"])
		end
		grp1:SetFullWidth(true)
	else
		grp1 = AceGUI:Create("SimpleGroup")
		grp1:SetLayout("Flow")
		grp1:SetFullWidth(true)
	end

	local grpWidth = 1
	local grpColNum = 5.01
	local grpRelWidth = grpWidth / grpColNum

	-- session control group - line 1
	GUI.lblSes = {}
	GUI.lblSes.startTime = GUI.AddButton("Label", L["Session Start"], "-")
	GUI.lblSes.startTime:SetRelativeWidth(grpRelWidth)
	grp1:AddChild(GUI.lblSes.startTime)

	GUI.lblSes.durationLap = GUI.AddButton("Label", L["Duration / Lap #"], "-")
	GUI.lblSes.durationLap:SetRelativeWidth(grpRelWidth)
	grp1:AddChild(GUI.lblSes.durationLap)

	GUI.lblSes.livGold = GUI.AddButton("Label", L["LIV / Gold"], "-")
	GUI.lblSes.livGold:SetRelativeWidth(grpRelWidth)
	grp1:AddChild(GUI.lblSes.livGold)

	GUI.lblSes.source = GUI.AddButton("Label", L["Price Source"], "-")
	GUI.lblSes.source:SetRelativeWidth(grpRelWidth)
	grp1:AddChild(GUI.lblSes.source)

	GUI.lblSes.itemsNw = GUI.AddButton("Label", "-", "-")
	GUI.lblSes.itemsNw:SetRelativeWidth(grpRelWidth)
	grp1:AddChild(GUI.lblSes.itemsNw)

	-- session control group - line 2
	GUI.infoSes = {}
	GUI.infoSes.startTime = GUI.AddButton("Label", "-", "-")
	GUI.infoSes.startTime:SetRelativeWidth(grpRelWidth)
	grp1:AddChild(GUI.infoSes.startTime)

	GUI.infoSes.durationLap = GUI.AddButton("Label", "-", "-")
	GUI.infoSes.durationLap:SetRelativeWidth(grpRelWidth)
	grp1:AddChild(GUI.infoSes.durationLap)

	GUI.infoSes.liv = GUI.AddButton("Label", "-", "-")
	GUI.infoSes.liv:SetRelativeWidth(grpRelWidth)

	GUI.infoSes.liv.label:SetJustifyH("RIGHT")
	GUI.labelFont, GUI.labelFontSize = GUI.infoSes.liv.label:GetFont()
	if not GUI.isShrinked then
		GUI.infoSes.liv.label:SetFont(GUI.labelFont, GUI.labelFontSize + 4)
	end
	grp1:AddChild(GUI.infoSes.liv)

	GUI.infoSes.source = GUI.AddButton("Label", "-", "-")
	GUI.infoSes.source:SetRelativeWidth(grpRelWidth)
	grp1:AddChild(GUI.infoSes.source)

	GUI.infoSes.itemsNw = GUI.AddButton("Label", "-", "-")
	GUI.infoSes.itemsNw:SetRelativeWidth(grpRelWidth)
	grp1:AddChild(GUI.infoSes.itemsNw)

	-- session control group - line 3
	GUI.btnSes = {}
	GUI.btnSes.stop = GUI.AddButton(L["Stop"], L["Stop the current session."])
	if GUI.isShrinked then
		GUI.btnSes.stop:SetText(L["STOP"])
	end

	GUI.btnSes.stop:SetRelativeWidth(grpRelWidth)
	GUI.btnSes.stop:SetCallback("OnClick",
		function()
			if (addon.db.realm.sesEnd == 0) then
				-- Stop Session
				addon.db.realm.sesEnd = time()
				addon:Printf(L["Stopping the session after %s. Click 'Save' or 'Wipe' to start a new one."],
					SecondsToTime(addon.db.realm.sesEnd - addon.db.realm.sesTime))
			else
				addon:Printf(L["Session has been stopped %s ago. Click 'Save' or 'Wipe' to start a new one."], SecondsToTime(time() - addon.db.realm.sesEnd))
			end
		end
	)
	grp1:AddChild(GUI.btnSes.stop)

	GUI.btnSes.lap = GUI.AddButton(L["Next"], nextLapPopupText)
	if GUI.isShrinked then
		GUI.btnSes.lap:SetText(L["NEXT"])
	end
	GUI.btnSes.lap:SetRelativeWidth(grpRelWidth)
	GUI.btnSes.lap:SetCallback("OnClick",
		function()
			if (addon.db.realm.sesEnd == 0) then
				addon.db.realm.lapTime = time()
				addon.db.realm.lapNum = addon.db.realm.lapNum + 1
				GUI.sesFilters = {}
				if (addon.db.global.nwSound ~= "") then
					addon.Options.PlaySound()
				end
				if (addon.db.global.lpInstanceReset) then
					ResetInstances()
					addon:Printf(L["Resetting the instance."])
				end
				addon:Printf(L["Finishing lap #%s with Looted Item Value %s."],
					addon.db.realm.lapNum - 1, GetCoinTextureString(addon.db.realm.sesMV))
				container:Reload()
			else
				addon:Printf(L["Can't start lap of a stopped session."])
			end
		end
	)
	grp1:AddChild(GUI.btnSes.lap)

	GUI.btnSes.gold = GUI.AddButton("Label", "-", "-")
	GUI.btnSes.gold:SetRelativeWidth(grpRelWidth)
	GUI.btnSes.gold.label:SetJustifyH("RIGHT")
	if not GUI.isShrinked then
		GUI.btnSes.gold.label:SetFont(GUI.labelFont, GUI.labelFontSize + 4)
	end
	grp1:AddChild(GUI.btnSes.gold)

	GUI.btnSes.save = GUI.AddButton(L["Save"], L["Save the current session and start a new one."])
	if GUI.isShrinked then
		GUI.btnSes.save:SetText(L["SAVE"])
	end
	GUI.btnSes.save:SetRelativeWidth(grpRelWidth)
	GUI.btnSes.save:SetCallback("OnClick",
		function()
			if (addon.db.global.nwSound ~= "") then
				addon.Options.PlaySound()
				GUI:DebugPrintf("Play %s", addon.db.global.nwSound)
			end
			addon.Data:SessionToDatabase()
			GUI.sesFilters = {}
			addon.Data:WipeSession()
			GUI:ShowInfo()
			addon:UpdateBroker()
			addon.Data:ResetCurrentTag(true)
			container:Reload()
		end
	)
	grp1:AddChild(GUI.btnSes.save)

	GUI.btnSes.wipe = GUI.AddButton(L["Wipe"], L["Wipe the current session data and start a new session."])
	if GUI.isShrinked then
		GUI.btnSes.wipe:SetText(L["WIPE"])
	end
	GUI.btnSes.wipe:SetRelativeWidth(grpRelWidth)
	GUI.btnSes.wipe:SetCallback("OnClick",
		function()
			if (addon.db.global.nwSound ~= "") then
				addon.Options.PlaySound()
			end
			GUI.sesFilters = {}
			addon.Data:WipeSession()
			GUI:ShowInfo()
			addon:UpdateBroker()
			container:Reload()
		end
	)
	grp1:AddChild(GUI.btnSes.wipe)

	container:AddChild(grp1)

	-- session data group
	local grp2

	if not GUI.isShrinked then
		grp2 = AceGUI:Create("SIDInlineGroup")
		grp2:SetLayout("Flow")
		if grp2.SetTitle then
			if addon.db.global.stackedView then
				grp2:SetTitle(L["Session Data"] .. L[" (stacked view)"])
			else
				grp2:SetTitle(L["Session Data"])
			end
		end
		grp2:SetFullWidth(true)
		grp2:SetFullHeight(true)
	else
		grp2 = AceGUI:Create("SimpleGroup")
		grp2:SetLayout("Flow")
		grp2:SetFullWidth(true)
		grp2:SetFullHeight(true)
	end

	if not GUI.isShrinked then
		local grp3 = AceGUI:Create("SimpleGroup")
		grp3:SetLayout("Flow")
		grp3:SetFullWidth(true)

		grpWidth = 1
		grpColNum = 4.01
		if not addon.db.global.shPlayer then
			grpColNum = grpColNum - 1
		end

		if not addon.db.global.shZone then
			grpColNum = grpColNum - 1
		end

		grpRelWidth = grpWidth / grpColNum

		-- session data group - line 1
		local i = GUI.AddButton("Label", L["Search"], "-")
		i:SetRelativeWidth(grpRelWidth)
		grp3:AddChild(i)

		i = GUI.AddButton("Label", L["Rarity"], "-")
		i:SetRelativeWidth(grpRelWidth)
		grp3:AddChild(i)

		if addon.db.global.shPlayer then
			i = GUI.AddButton("Label", L["Player"], "-")
			i:SetRelativeWidth(grpRelWidth)
			grp3:AddChild(i)
		end

		if addon.db.global.shZone then
			i = GUI.AddButton("Label", L["Zone"], "-")
			i:SetRelativeWidth(grpRelWidth)
			grp3:AddChild(i)
		end
		grp2:AddChild(grp3)

		grp3 = AceGUI:Create("SimpleGroup")
		grp3:SetLayout("Flow")
		grp3:SetFullWidth(true)

		-- session data group - line 2
		i = GUI.AddButton("SIDEditBox", "", "-")
		i:SetRelativeWidth(grpRelWidth)
		i.button:SetScript("OnClick", function(f)
			local editbox = f.obj.editbox
			editbox:SetText("")
			editbox:SetCursorPosition(0)
			editbox:ClearFocus()
			f.obj:DisableButton(true)
			f.obj:Fire("OnTextChanged", "")
		end)
		i.button:SetText(L["CLR"])
		i:SetCallback("OnTextChanged",
			function(f, _, value)
				value = value:trim()
				if value == "" then
					GUI.sesFilters.name = nil
				else
					GUI.sesFilters.name = value
					f:DisableButton(false)
				end
				GUI.dataSes:SetContent(private.GetSessionItemData(GUI.sesFilters))
				private.GetSessionInfoData(GUI.sesFilters)
			end
		)
		grp3:AddChild(i)

		i = GUI.AddButton("Dropdown", L["Rarity"], "-")
		i:SetRelativeWidth(grpRelWidth)
		i:SetList(GUI.rarityList)
		i:SetCallback("OnValueChanged",
			function(_, _, key)
				if key > 0 then
					GUI.sesFilters.rarity = key
				else
					GUI.sesFilters.rarity = nil
				end
				GUI.dataSes:SetContent(private.GetSessionItemData(GUI.sesFilters))
				private.GetSessionInfoData(GUI.sesFilters)
			end
		)
		grp3:AddChild(i)

		if addon.db.global.shPlayer then
			i = GUI.AddButton("Dropdown", L["Player"], "-")
			i:SetRelativeWidth(grpRelWidth)
			i:SetList(GUI.playerListFromItems(addon.sesItems))
			local value = "all"
			i:SetText(i.list[value])
			i.value = value
			i:SetCallback("OnValueChanged",
				function(_, _, val)
					if val == "all" then
						GUI.sesFilters.player = nil
					else
						GUI.sesFilters.player = val
					end
					GUI.dataSes:SetContent(private.GetSessionItemData(GUI.sesFilters))
					private.GetSessionInfoData(GUI.sesFilters)
				end
			)
			grp3:AddChild(i)
		end

		if addon.db.global.shZone then
			i = GUI.AddButton("Dropdown", L["Zone"], "-")
			i:SetRelativeWidth(grpRelWidth)
			i:SetList(GUI.zoneListFromItems(addon.sesItems))
			local value = "all"
			i:SetText(i.list[value])
			i.value = value
			i:SetCallback("OnValueChanged",
				function(_, _, val)
					if val == "all" then
						GUI.sesFilters.zone = nil
					else
						GUI.sesFilters.zone = val
					end
					GUI.dataSes:SetContent(private.GetSessionItemData(GUI.sesFilters))
					private.GetSessionInfoData(GUI.sesFilters)
				end
			)
			grp3:AddChild(i)
		end
		grp2:AddChild(grp3)
	end

	-- session data group - item list
	local grp4 = AceGUI:Create("SimpleGroup")
	grp4:SetLayout("Fill")
	grp4:SetFullWidth(true)
	grp4:SetFullHeight(true)

	local titles = {
		{text=L["Item"] .. flNotTheRealItemString, relWidth=GUI.TITLEWIDTH_ITEM},
		{text=L["Value   "], relWidth=-GUI.TITLEWIDTH_VALUE},
		{text=L["Qty "], relWidth=-GUI.TITLEWIDTH_QTY},
		{text=L["Player"], relWidth=GUI.TITLEWIDTH_PLAYER},
		{text=L["Zone"], relWidth=GUI.TITLEWIDTH_ZONE},
		{text=L["Time"], relWidth=GUI.TITLEWIDTH_TIME},
	}
	local colSpan = 0
	local colSpan2 = 6
	if (not addon.db.global.shPlayer) then
		colSpan = colSpan + GUI.TITLEWIDTH_PLAYER - GUI.TITLEWIDTH_GAP
		titles[4] = {text="", relWidth=GUI.TITLEWIDTH_GAP}
		colSpan2 = colSpan2 - 1
	end
	if (not addon.db.global.shZone) then
		colSpan = colSpan + GUI.TITLEWIDTH_ZONE - GUI.TITLEWIDTH_GAP
		titles[5] = {text="", relWidth=GUI.TITLEWIDTH_GAP}
		colSpan2 = colSpan2 - 1
	end
	if (not addon.db.global.shTime) then
		colSpan = colSpan + GUI.TITLEWIDTH_TIME - GUI.TITLEWIDTH_GAP
		titles[6] = {text="", relWidth=GUI.TITLEWIDTH_GAP}
		colSpan2 = colSpan2 - 1
	end
	titles[1].relWidth = GUI.TITLEWIDTH_ITEM  + colSpan / colSpan2
	titles[2].relWidth = - GUI.TITLEWIDTH_VALUE - colSpan / colSpan2
	titles[3].relWidth = - GUI.TITLEWIDTH_QTY - colSpan / colSpan2
	if titles[4].text ~= "" then
		titles[4].relWidth = GUI.TITLEWIDTH_PLAYER + colSpan / colSpan2
	end
	if titles[5].text ~= "" then
		titles[5].relWidth = GUI.TITLEWIDTH_ZONE + colSpan / colSpan2
	end
	if titles[6].text ~= "" then
		titles[6].relWidth = GUI.TITLEWIDTH_TIME + colSpan / colSpan2
	end

	container.titleStatusText = {
		L["Leftclick to sort this column / Rightclick to show all columns"],
		L["Leftclick to sort this column"],
		L["Leftclick to sort this column"],
		L["Leftclick to sort and Rightclick to hide Player column"],
		L["Leftclick to sort and RightClick to hide Zone column"],
		L["Leftclick to sort and RightClick to hide Time column"],
		}

	container.statusText = L["Alt-Rightclick to delete the item"]

	local eventHandlers = {
		OnEnter = GUI.DataFrame_OnEnter,
		OnLeave = GUI.DataFrame_OnLeave,
		OnClick = function(_, data, button)
			if not data then return end
			-- for item debugging
			if button == "LeftButton" and IsShiftKeyDown() then
				GUI:PrintItemStrings(data.link)
			end
			-- for item debugging
			if button == "RightButton" and IsRightShiftKeyDown() then
				if testitem then
					local i = addon.Item:GetItemID(data.link)
					if i then
						testitem(data.link,i,1)
						addon.Data:InsertItem(data.link, 1, addon.Item:GetItemQuality(data.link,data.link))
						addon.Data:UpdateSession()
						addon.GUI:UpdateSessionData(true)
					end
				end
			end
			if button == "RightButton" and not IsAltKeyDown() then
				if type(data) == "number" then
					if data == 4 then
						addon.db.global.shPlayer = false
						container:Reload()
					end
					if data == 1 then
						addon.db.global.shPlayer = true
						addon.db.global.shZone = true
						addon.db.global.shTime = true
						container:Reload()
					end
					if data == 5 then
						addon.db.global.shZone = false
						container:Reload()
					end
					if data == 6 then
						addon.db.global.shTime = false
						GUI.dataSes:SetSortTitle(1)
						container:Reload()
					end
				end
			end
			-- purge the line/item
			if button == "RightButton" and IsAltKeyDown() then
				local qty = addon.Data:PurgeItem(data, addon.sesItems)
				if (qty > 0) then
					addon.Data:UpdateSession()
					GUI:UpdateSessionData(true)
					addon:Printf(L["%sx %s purged."], qty, data.link)
				end
			end
		end,
	}

	GUI.dataSes = AceGUI:Create("SIDData")
	GUI.dataSes:DoRedraw(false)
	-- GUI.dataSes:SetTextSize(11)
	GUI.dataSes:SetEventHandlers(eventHandlers)
	GUI.dataSes:SetTitles(titles)
	if addon.db.global.shTime then
		GUI.dataSes:SetSortTitle(-6)
	else
		GUI.dataSes:SetSortTitle(1)
	end
	grp4:AddChild(GUI.dataSes)

	grp2:AddChild(grp4)
	container:AddChild(grp2)

	GUI.dataSes:SetContent(private.GetSessionItemData(GUI.sesFilters))
	GUI.dataSes:DoRedraw(true)

	private.GetSessionInfoData(GUI.sesFilters)
	addon:UpdateBroker()

	return container
end

-- prepare/update session item data
function private.GetSessionItemData(lfilters)
	GUI:DebugPrintf("GetSessionItemData()")
	if GUI.view ~= "session" or not GUI.display then
		GUI:DebugPrintf("... nothing done.")
		return
	end

	return GUI.GetItemData(addon.sesItems, lfilters)
end

-- prepare/update session info data
function private.GetSessionInfoData(filters)
	GUI:DebugPrintf("GetSessionInfoData()")
	if GUI.view ~= "session" or not GUI.display then
		GUI:DebugPrintf("... nothing done.")
		return
	end

	local mvFilter = 0
	local nwFilter = 0
	local itsSum = 0
	local itsFilter = 0
	local itSum = 0
	local itFilter = 0
	local isFiltered = false

	if (not filters.name and not filters.rarity and not filters.player and not filters.time and not filters.zone) then
		GUI:DebugPrintf("... with no filters")
	else
		GUI:DebugPrintf("... with filters")
		isFiltered = true
		for link, data in pairs(addon.sesItems) do
			if #data.drops > 0 then
				itsSum = itsSum + 1
			end
			if #data.drops > 0 and not GUI.IsItemFiltered(link, data, filters) then
				itsFilter = itsFilter + 1
				for _, record in ipairs(data.drops) do
					itSum = itSum + record.quantity
					if not GUI.IsRecordFiltered(record, filters) then
						itFilter = itFilter + record.quantity
						local marketValue = addon.Data:GetItemValue(record.link) or 0
						mvFilter = mvFilter + (marketValue * record.quantity)
						if GUI:ToCopper(addon.db.global.minNoteworthy) then
							if (marketValue > GUI:ToCopper(addon.db.global.minNoteworthy)) then
								nwFilter = nwFilter + record.quantity
							end
						end
					end
				end
			end
		end
	end

	if (isFiltered or (itsSum ~= itsFilter) or (itSum ~= itFilter)) then
		if ((GUI.infoSes) and (GUI.btnSes)) then
			-- Session Start Time
			GUI.lblSes.startTime:SetText(L["Data Source"])
			GUI.infoSes.startTime:SetText(ITEM_QUALITY_COLORS[5].hex ..
				L["Selection"]
				.. FONT_COLOR_CODE_CLOSE)
			-- Duration / Lap #
			GUI.lblSes.durationLap:SetText("")
			GUI.infoSes.durationLap:SetText("")
			-- LIV / Gold
			GUI.lblSes.livGold:SetText(L["LIV"])
			GUI.infoSes.liv:SetText(GUI:FromCopper(mvFilter))
			GUI.infoSes.liv.label:SetAllPoints()
			GUI.btnSes.gold:SetText("")
			-- Price Source
			local mvSourcesList = addon.Item:GetPriceSources() or {}
			if mvSourcesList[addon.db.global.mvSource] then
				GUI.infoSes.source:SetText(ITEM_QUALITY_COLORS[6].hex ..
					addon.db.global.mvSource
					.. FONT_COLOR_CODE_CLOSE)
			else
				GUI.infoSes.source:SetText(ITEM_QUALITY_COLORS[0].hex ..
					"(" .. addon.db.global.mvSource .. ")"
					.. FONT_COLOR_CODE_CLOSE)
			end
			-- Items / Noteworthy
			GUI.lblSes.itemsNw:SetText(ITEM_QUALITY_COLORS[addon.db.global.minRarity].hex .. L["Items"] .. FONT_COLOR_CODE_CLOSE ..
									" / " .. ">" .. addon.db.global.minNoteworthy)
			if GUI.isShrinked then
				GUI.lblSes.itemsNw:SetText(ITEM_QUALITY_COLORS[addon.db.global.minRarity].hex .. L["Items"] .. FONT_COLOR_CODE_CLOSE)
			end
			GUI.infoSes.itemsNw:SetText(ITEM_QUALITY_COLORS[6].hex ..
				itFilter .. " / " .. nwFilter
				..  FONT_COLOR_CODE_CLOSE)
		end
	else
		if ((GUI.infoSes) and (GUI.btnSes)) then
			-- Session Start Time
			GUI.lblSes.startTime:SetText(L["Session Start"])
			if GUI.isShrinked then
				GUI.lblSes.startTime:SetText(L["Start"])
			end

			GUI.infoSes.startTime:SetText(ITEM_QUALITY_COLORS[6].hex ..
				GUI:TimeToString(addon.db.realm.sesTime)
				.. FONT_COLOR_CODE_CLOSE)
			-- Duration / Lap #
			if (addon.db.realm.sesEnd == 0) then
				GUI.lblSes.durationLap:SetText(L["Duration / Lap #"])
				GUI.infoSes.durationLap:SetText(ITEM_QUALITY_COLORS[6].hex ..
					SecondsToTime(time() - addon.db.realm.sesTime) .. " / " .. addon.db.realm.lapNum
						.. FONT_COLOR_CODE_CLOSE)
				if GUI.isShrinked then
					GUI.lblSes.durationLap:SetText(L["Dur/Lap #"])
					local z = time() - addon.db.realm.sesTime
					if (z > 60) then
						z = floor(z / 60) * 60
					end
					GUI.infoSes.durationLap:SetText(ITEM_QUALITY_COLORS[6].hex ..
						GUI.ShrinkedTime(SecondsToTime(z)) .. "/" .. addon.db.realm.lapNum
						.. FONT_COLOR_CODE_CLOSE)
				end
			else
				GUI.lblSes.durationLap:SetText(L["Duration"])
				GUI.infoSes.durationLap:SetText(ITEM_QUALITY_COLORS[6].hex ..
				SecondsToTime(addon.db.realm.sesEnd - addon.db.realm.sesTime)
				.. FONT_COLOR_CODE_CLOSE)
			end
			-- Looted Item Value / Gold
			if GUI.isShrinked then
				GUI.lblSes.livGold:SetText(L["LIV/Gold"])
			else
				GUI.lblSes.livGold:SetText(L["LIV / Gold"])
			end
			GUI.infoSes.liv:SetText(GUI:FromCopper(addon.db.realm.sesMV))
			GUI.infoSes.liv.label:SetAllPoints()
			GUI.btnSes.gold:SetText(GUI:FromCopper(addon.db.realm.sesGold))
			GUI.btnSes.gold.label:SetAllPoints()
			-- Price Source
			if GUI.isShrinked then
				GUI.lblSes.source:SetText(L["P/Source"])
			else
				GUI.lblSes.source:SetText(L["Price Source"])
			end
			local mvSourcesList = addon.Item:GetPriceSources() or {}
			if mvSourcesList[addon.db.global.mvSource] then
				GUI.infoSes.source:SetText(ITEM_QUALITY_COLORS[6].hex ..
					addon.db.global.mvSource
					.. FONT_COLOR_CODE_CLOSE)
				if GUI.isShrinked then
					local s = addon.db.global.mvSource
					s = gsub(s, "TSM:", "")
					s = gsub(s, "UJ:", "")
					s = gsub(s, "^UJ:", "")
					if s:match("^(........)") then
						s = s:match("^(........)")
					end
					GUI.infoSes.source:SetText(ITEM_QUALITY_COLORS[6].hex .. s .. FONT_COLOR_CODE_CLOSE)
				end
			else
				GUI.infoSes.source:SetText(ITEM_QUALITY_COLORS[0].hex ..
					"(" .. addon.db.global.mvSource .. ")"
					.. FONT_COLOR_CODE_CLOSE)
			end
			-- Items / Noteworthy
			GUI.lblSes.itemsNw:SetText(ITEM_QUALITY_COLORS[addon.db.global.minRarity].hex .. L["Items"] .. FONT_COLOR_CODE_CLOSE ..
									" / " .. ">" .. addon.db.global.minNoteworthy)
			if GUI.isShrinked then
				GUI.lblSes.itemsNw:SetText(ITEM_QUALITY_COLORS[addon.db.global.minRarity].hex .. L["Items"] .. FONT_COLOR_CODE_CLOSE)
			end
			GUI.infoSes.itemsNw:SetText(ITEM_QUALITY_COLORS[6].hex ..
				addon.db.realm.sesNumItems .. " / " .. addon.db.realm.sesNumNW
				..  FONT_COLOR_CODE_CLOSE)
		end
	end
end

-- prepare/update session data
function GUI:UpdateSessionData(sesItemsUpdated)
	-- GUI:DebugPrintf("UpdateSessionData(" .. tostring(sesItemsUpdated) .. ")")
	if GUI.view == "session" and GUI.display then
		if (sesItemsUpdated) then
			GUI.dataSes:SetContent(private.GetSessionItemData(GUI.sesFilters))
		end
		private.GetSessionInfoData(GUI.sesFilters)
	end
	addon:UpdateBroker()

	-- auto adapt data + info view
	if GUI.display then
		if GUI.view == "session" then
			GUI:DebugPrintf("Adapt Session")
			if GUI.labelFont and GUI.labelFontSize and GUI.container then
				if GUI.adapt.width and GUI.adapt.width >= 600 then
					GUI.infoSes.liv.label:SetFont(GUI.labelFont, GUI.labelFontSize + 4)
					GUI.btnSes.gold.label:SetFont(GUI.labelFont, GUI.labelFontSize + 4)
					if GUI.TITLEWIDTH_ITEM ~= 0.31 then
						if GUI.adapt.shPlayer ~= nil then
							addon.db.global.shPlayer = GUI.adapt.shPlayer
							GUI.adapt.shPlayer = nil
						end
						if GUI.adapt.shZone ~= nil then
							addon.db.global.shZone = GUI.adapt.shZone
							GUI.adapt.shZone = nil
						end
						GUI.TITLEWIDTH_ITEM = 0.31
						GUI.TITLEWIDTH_VALUE = 0.17
						GUI.TITLEWIDTH_QTY = 0.04
						GUI.TITLEWIDTH_PLAYER = 0.10
						GUI.TITLEWIDTH_ZONE = 0.20
						GUI.TITLEWIDTH_TIME = 0.18
						GUI:Shrink(false)
						-- GUI:Printf("Session Reload >= 600")
						GUI.container:Reload()
					end
				elseif GUI.adapt.width and GUI.adapt.width >= 500 then
					GUI.infoSes.liv.label:SetFont(GUI.labelFont, GUI.labelFontSize + 2)
					GUI.btnSes.gold.label:SetFont(GUI.labelFont, GUI.labelFontSize + 2)
					if GUI.TITLEWIDTH_ITEM ~= 0.35 then
						if GUI.adapt.shPlayer == nil then
							GUI.adapt.shPlayer = addon.db.global.shPlayer
							addon.db.global.shPlayer = false
						end
						if GUI.adapt.shZone ~= nil then
							addon.db.global.shZone = GUI.adapt.shZone
							GUI.adapt.shZone = nil
						end
						GUI.TITLEWIDTH_ITEM = 0.35
						GUI.TITLEWIDTH_VALUE = 0.21
						GUI.TITLEWIDTH_QTY = 0.04
						GUI.TITLEWIDTH_PLAYER = 0.10
						GUI.TITLEWIDTH_ZONE = 0.17
						GUI.TITLEWIDTH_TIME = 0.13
						-- GUI:Printf("Session Reload >= 500")
						GUI:Shrink(false)
						GUI.container:Reload()
					end
				else
					GUI.infoSes.liv.label:SetFont(GUI.labelFont, GUI.labelFontSize)
					GUI.btnSes.gold.label:SetFont(GUI.labelFont, GUI.labelFontSize)
					if GUI.TITLEWIDTH_ITEM ~= 0.40 then
						if GUI.adapt.shPlayer == nil then
							GUI.adapt.shPlayer = addon.db.global.shPlayer
							addon.db.global.shPlayer = false
						end
						if GUI.adapt.shZone == nil then
							GUI.adapt.shZone = addon.db.global.shZone
							addon.db.global.shZone = false
						end
						GUI.TITLEWIDTH_ITEM = 0.40
						GUI.TITLEWIDTH_VALUE = 0.22
						GUI.TITLEWIDTH_QTY = 0.04
						GUI.TITLEWIDTH_PLAYER = 0.10
						GUI.TITLEWIDTH_ZONE = 0.13
						GUI.TITLEWIDTH_TIME = 0.11
						-- GUI:Printf("Session Reload")
						GUI:Shrink(true)
						GUI.container:Reload()
					end
				end
			end
		end

		if GUI.view == "history" then
			GUI:DebugPrintf("Adapt History")
			if GUI.labelFont and GUI.labelFontSize and GUI.container then
				if GUI.adapt.width and GUI.adapt.width >= 600 then
					GUI.infoHis.liv.label:SetFont(GUI.labelFont, GUI.labelFontSize + 4)
					if GUI.TITLEWIDTH_ITEM ~= 0.31 then
						if GUI.adapt.shPlayer ~= nil then
							addon.db.global.shPlayer = GUI.adapt.shPlayer
							GUI.adapt.shPlayer = nil
						end
						if GUI.adapt.shZone ~= nil then
							addon.db.global.shZone = GUI.adapt.shZone
							GUI.adapt.shZone = nil
						end
						GUI.TITLEWIDTH_ITEM = 0.31
						GUI.TITLEWIDTH_VALUE = 0.17
						GUI.TITLEWIDTH_QTY = 0.04
						GUI.TITLEWIDTH_PLAYER = 0.10
						GUI.TITLEWIDTH_ZONE = 0.20
						GUI.TITLEWIDTH_TIME = 0.18
						-- GUI:Printf("History Reload >= 600")
						GUI.container:Reload()
					end
				elseif GUI.adapt.width and GUI.adapt.width >= 500 then
					GUI.infoHis.liv.label:SetFont(GUI.labelFont, GUI.labelFontSize + 2)
					if GUI.TITLEWIDTH_ITEM ~= 0.35 then
						if GUI.adapt.shPlayer == nil then
							GUI.adapt.shPlayer = addon.db.global.shPlayer
							addon.db.global.shPlayer = false
						end
						if GUI.adapt.shZone ~= nil then
							addon.db.global.shZone = GUI.adapt.shZone
							GUI.adapt.shZone = nil
						end
						GUI.TITLEWIDTH_ITEM = 0.35
						GUI.TITLEWIDTH_VALUE = 0.21
						GUI.TITLEWIDTH_QTY = 0.04
						GUI.TITLEWIDTH_PLAYER = 0.10
						GUI.TITLEWIDTH_ZONE = 0.17
						GUI.TITLEWIDTH_TIME = 0.13
						-- GUI:Printf("History Reload >= 500")
						GUI.container:Reload()
					end
				else
					GUI.infoHis.liv.label:SetFont(GUI.labelFont, GUI.labelFontSize)
					if GUI.TITLEWIDTH_ITEM ~= 0.40 then
						if GUI.adapt.shPlayer == nil then
							GUI.adapt.shPlayer = addon.db.global.shPlayer
							addon.db.global.shPlayer = false
						end
						if GUI.adapt.shZone == nil then
							GUI.adapt.shZone = addon.db.global.shZone
							addon.db.global.shZone = false
						end
						GUI.TITLEWIDTH_ITEM = 0.40
						GUI.TITLEWIDTH_VALUE = 0.22
						GUI.TITLEWIDTH_QTY = 0.04
						GUI.TITLEWIDTH_PLAYER = 0.10
						GUI.TITLEWIDTH_ZONE = 0.13
						GUI.TITLEWIDTH_TIME = 0.11
						-- GUI:Printf("History Reload")
						GUI.container:Reload()
					end
				end
			end
		end
	end

	-- auto shrink frame
	if GUI.display then
		if GUI.view == "session" then
			if GUI.isShrinked then
				if not (GUI.lastWidth and GUI.container.frame:GetWidth() == GUI.lastWidth) then
					-- GUI:Printf("Shrink/SetPoints")
					GUI.container.border:SetPoint("TOPLEFT", 1, - 8)
					GUI.lastWidth = GUI.container.frame:GetWidth()
				end
			end
		end
	end
end

-- EOF

